@@ -1,7 +1,7 @@
 # Traffic Monitor Detector Raw Data (TMD)
 # modify from LPD
-# ver:0.0.1
-# 2020/04/30
+# ver:0.0.2
+# 2021/04/24
 # parsing Traffic Monitor Detector
 # hardware:(Batman-201): ISK IWR6843 ES2.0
 # company: Joybien Technologies: www.joybien.com
@@ -9,11 +9,14 @@
 #===========================================
 # output: V6,V7,V8,v9 Raw data
 # v0.0.1 : 2020/04/30 release
+# v0.1.0 : 2021/04/24 release
+#    added DataFrame 
 
 import serial
 import time
 import struct
-#import numpy as np
+import pandas as pd
+import numpy as np
 
 class header:
 	version = 0
@@ -37,7 +40,28 @@ class TrafficMD:
 	port = ""
 	hdr = header
 
-	# add for PMB interal use
+	# provide csv file dataframe
+	# real-time 
+	v6_col_names_rt = ['fN','type','range','azimuth','elv' ,'doppler','sx', 'sy', 'sz'] #ok
+	v7_col_names_rt = ['fN','type','posX','posY','velX','velY','accX','accY','posZ','velZ','accZ','tid']
+	v8_col_names_rt = ['fN','type','targetID']
+	v9_col_names_rt = ['fN','type','snr','noise']
+	
+	# read from file for trace point clouds
+	fileName = ''
+	v6_col_names = ['time','fN','type','range','azimuth','elv' ,'doppler','sx', 'sy', 'sz']
+	v7_col_names = ['time','fN','type','posX','posY','velX','velY','accX','accY','posZ','velZ','accZ','tid']
+	v8_col_names = ['time','fN','type','targetID']
+	v9_col_names = ['fN','type','snr','noise']
+	sim_startFN = 0
+	sim_stopFN  = 0 
+	
+	v6simo = []
+	v7simo = []
+	v8simo = []
+	v9simo = []
+	
+	# add for interal use
 	tlvLength = 0
 	numOfPoints = 0
 	# for debug use 
@@ -49,6 +73,7 @@ class TrafficMD:
 		self.port = port
 		print("(jb)Traffic Monitor Detector(TMD) raw Data lib initial")
 		print("(jb)For Hardware:Batman-201(ISK)")
+		print("(jb)Version: v0.1.0")
 		print("(jb)Hardware: IWR-6843 ES2.0")
 		print("(jb)Firmware: TMD")
 		print("(jb)UART Baud Rate:921600")
@@ -143,6 +168,18 @@ class TrafficMD:
 		
 		return unitByte,stateString, sbyte, dataByte,count, int(nPoint)
 		
+	def list2df(self,dck,l6,l7,l8,l9):
+		#print("---------------list2df: v6----------------")
+		#print(l6)
+		
+		ll6 = pd.DataFrame(l6,columns=self.v6_col_names_rt)
+		ll7 = pd.DataFrame(l7,columns=self.v7_col_names_rt)
+		ll8 = pd.DataFrame(l8,columns=self.v8_col_names_rt)
+		ll9 = pd.DataFrame(l9,columns=self.v9_col_names_rt)
+		#print("------ll6---------list2df: v6----------------")
+		#print(ll6)
+		return (dck,ll6,ll7,ll8,ll9)
+
 #
 # TLV: Type-Length-Value
 # read TLV data
@@ -158,7 +195,7 @@ class TrafficMD:
 #  v8: Target Index information
 #  v9: DPIF Point Cloud Side Infomation
 #
-	def tlvRead(self,disp):
+	def tlvRead(self,disp,df = None):
 		#print("---tlvRead---")
 		#ds = dos
 		typeList = [6,7,8,9]
@@ -175,12 +212,17 @@ class TrafficMD:
 		v7 = ([])
 		v8 = ([])
 		v9 = ([])
+		v6df = ([])
+		v7df = ([])
+		v8df = ([])
+		v9df = ([])
 	
 		while True:
 			try:
 				ch = self.port.read()
 			except:
-				return (False,v6,v7,v8,v9)
+				#return (False,v6,v7,v8,v9)
+				return self.list2df(False,v6,v7,v8,v9) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
 			#print(str(ch))
 			if lstate == 'idle':
 				#print(self.magicWord)
@@ -188,7 +230,6 @@ class TrafficMD:
 					#print("*** magicWord:"+ "{:02x}".format(ord(ch)) + ":" + str(idx))
 					idx += 1
 					if idx == 8:
-						
 						idx = 0
 						lstate = 'header'
 						rangeProfile = b""
@@ -197,7 +238,8 @@ class TrafficMD:
 					#print("not: magicWord state:")
 					idx = 0
 					rangeProfile = b""
-					return (False,v6,v7,v8,v9)
+					#return (False,v6,v7,v8,v9)
+					return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
 		
 			elif lstate == 'header':
 				sbuf += ch
@@ -216,14 +258,16 @@ class TrafficMD:
 					except:
 						if self.dbg == True:
 							print("(Header)Improper TLV structure found: ")
-						return (False,v6,v7,v8,v9)
+						#return (False,v6,v7,v8,v9)
+						return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
 					
 					if disp == True:  
 						self.headerShow()
 					
 					tlvCount = self.hdr.numTLVs
 					if self.hdr.numTLVs == 0:
-						return (False,v6,v7,v8,v9)
+						#return (False,v6,v7,v8,v9)
+						return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
 						
 					if self.sm == True:
 						print("(Header)")
@@ -235,7 +279,8 @@ class TrafficMD:
 				elif idx > 48:
 					idx = 0
 					lstate = 'idle'
-					return (False,v6,v7,v8,v9)
+					#return (False,v6,v7,v8,v9)
+					return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
 					
 			elif lstate == 'TL': #TLV Header type/length
 				sbuf += ch
@@ -253,15 +298,17 @@ class TrafficMD:
 							idx = 0
 							lstate = 'idle'
 							self.port.flushInput()
-							return (False,v6,v7,v8,v9)
+							#return (False,v6,v7,v8,v9)
+							return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
 							
 					except:
 						print("TL unpack Improper Data Found:")
 						if self.dbg == True:
 							print("TL unpack Improper Data Found:")
 						self.port.flushInput()
-						return (False,v6,v7,v8,v9)
+						#return (False,v6,v7,v8,v9)
 					
+						return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
 					
 					unitByteCount,lstate ,plen ,dataBytes,lenCount, numOfPoints = self.tlvTypeInfo(ttype,self.tlvLength,disp)
 					#print("dataBytes={:d} lenCount={:d}".format(dataBytes , lenCount))
@@ -280,12 +327,21 @@ class TrafficMD:
 						#print(":".join("{:02x}".format(c) for c in sbuf))
 						(r,a,e,d) = struct.unpack('4f', sbuf)
 						#print("({:2d}:{:4d})(idx:({:4d}) elv:{:.4f} azimuth:{:.4f} doppler:{:.4f} range:{:.4f}".format(numOfPoints,lenCount,idx,e,a,d,r))
-						v6.append((r,a,e,d))
+						if (df == 'DataFrame'):
+							sz  = r * np.sin(e)
+							sx  = r * np.cos(e) * np.sin(a)
+							sy  = r * np.cos(e) * np.cos(a)
+							#fieldNames = ['time','frameNum','type','range/px','azimuth/py','elv/vx','doppler/vy','sx/accX','sy/accY','sz/pz','na/vz','na/accZ','na/ID']
+							v6df.append((self.hdr.frameNumber,'v6',r,a,e,d,sx,sy,sz))
+						else:
+							v6.append((r,a,e,d))
+							
 						sbuf = b""
 					except:
 						if self.dbg == True:
 							print("(6.1)Improper Type 6 Value structure found: ")
-						return (False,v6,v7,v8,v9)
+						#return (False,v6,v7,v8,v9)
+						return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
 					
 				if idx == lenCount:
 					if disp == True:
@@ -296,7 +352,9 @@ class TrafficMD:
 						lstate = 'idle'
 						if self.sm == True:
 							print("(V6:tlvCnt={:d})=>(idle) :true".format(tlvCount))
-						return (True,v6,v7,v8,v9)
+						#return (True,v6,v7,v8,v9)
+						print("(lib)v6=============SaveAppend==============")
+						return self.list2df(True,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (True,v6,v7,v8,v9)
 						
 					else: # Go to TL to get others type value
 						lstate = 'TL' #'tlTL'
@@ -307,7 +365,8 @@ class TrafficMD:
 					idx = 0
 					sbuf = b""
 					lstate = 'idle'
-					return (False,v6,v7,v8,v9)
+					#return (False,v6,v7,v8,v9)
+					return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
 					
 			elif lstate == 'V9':  
 				sbuf += ch
@@ -317,12 +376,19 @@ class TrafficMD:
 						#print(":".join("{:02x}".format(c) for c in sbuf))
 						(snr,noise) =  struct.unpack('2h', sbuf)
 						#print("lenCnt = ({:d}) snr:{:.4f} noise:{:.4f}".format(lenCount,snr,noise))
-						v9.append((snr,noise))
+						
+						if (df == 'DataFrame'):
+							v9df.append((self.hdr.frameNumber,'v9',snr,noise))
+						else:
+							v9.append((snr,noise))
+							
 						sbuf = b""
 					except:
 						if self.dbg == True:
 							print("(7)Improper Type 9 Value structure found: ")
-						return (False,v6,v7,v8,v9)
+						#return (False,v6,v7,v8,v9)
+						return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
+						
 				if idx >= lenCount:
 					if disp == True:
 						print("count= v9[{:d}]".format(len(v9)))
@@ -332,7 +398,9 @@ class TrafficMD:
 						lstate = 'idle'
 						if self.sm == True:
 							print("(V9)=>(idle) :true")
-						return (True,v6,v7,v8,v9)
+						#return (True,v6,v7,v8,v9)
+						return self.list2df(True,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (True,v6,v7,v8,v9)
+						
 					else: # Go to TL to get others type value
 						lstate = 'TL'
 						idx = 0
@@ -343,7 +411,8 @@ class TrafficMD:
 					idx = 0 
 					lstate = 'idle'
 					sbuf = b""
-					return (False,v6,v7,v8,v9)
+					#return (False,v6,v7,v8,v9)
+					return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
 					
 			elif lstate == 'V7':
 				sbuf += ch
@@ -352,13 +421,18 @@ class TrafficMD:
 					#print("V7:dataBytes({:d}) lenCount({:d}) index:{:d}".format(dataBytes,lenCount,idx))
 					try:
 						(tid,posX,posY,velX,velY,accX,accY,posZ,velZ,accZ) = struct.unpack('I9f', sbuf) 
-						v7.append((tid,posX,posY,velX,velY,accX,accY,posZ,velZ,accZ))
-						#print("tid = ({:d}) ,posX:{:.4f} posY:{:.4f} posZ:{:.4f}".format(tid,posX,posY,posZ))
+						#fieldNames = ['time','frameNum','type','range/px','azimuth/py','elv/vx','doppler/vy','sx/accX','sy/accY','sz/pz','na/vz','na/accZ','na/ID']
+						if (df == 'DataFrame' ):
+							v7df.append((self.hdr.frameNumber,'v7',posX,posY,velX,velY,accX,accY,posZ,velZ,accZ,tid))
+						else:
+							v7.append((tid,posX,posY,velX,velY,accX,accY,posZ,velZ,accZ))
+							#print("tid = ({:d}) ,posX:{:.4f} posY:{:.4f} posZ:{:.4f}".format(tid,posX,posY,posZ))
 						sbuf = b""
 					except:
 						if self.dbg == True:
 							print("(7)Improper Type 7 Value structure found: ")
-						return (False,v6,v7,v8,v9)
+						#return (False,v6,v7,v8,v9)
+						return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
 						
 				if idx >= lenCount:
 					if disp == True:
@@ -369,7 +443,8 @@ class TrafficMD:
 						lstate = 'idle'
 						if self.sm == True:
 							print("(V7)=>(idle) :true")
-						return (True,v6,v7,v8,v9)
+						#return (True,v6,v7,v8,v9)
+						return self.list2df(True,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (True,v6,v7,v8,v9)
 						
 					else: # Go to TL to get others type value
 						lstate = 'TL'
@@ -381,7 +456,8 @@ class TrafficMD:
 					idx = 0 
 					lstate = 'idle'
 					sbuf = b""
-					return (False,v6,v7,v8,v9)
+					#return (False,v6,v7,v8,v9)
+					return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
 				
 			elif lstate == 'V8':
 				idx += 1
@@ -394,14 +470,85 @@ class TrafficMD:
 					lstate = 'idle'
 					if self.sm == True:
 						print("(V8:{:d})=>(idle)".format(tlvCount))
-					return (True,v6,v7,v8,v9)
+						
+					if (df == 'DataFrame'):
+						v8o = [self.hdr.frameNumber,'v8']
+						v8df = v8o.extend(v8)
+					#return (True,v6,v7,v8,v9)
+					return self.list2df(True,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (True,v6,v7,v8,v9)
 				
 				if idx > lenCount:
 					sbuf = b""
 					idx = 0
 					lstate = 'idle'
-					return (False,v6,v7,v8,v9)
+					#return (False,v6,v7,v8,v9)
+					return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
 
-
+	def getRecordData(self,frameNum):
+		s_fn = frameNum + self.sim_startFN
+		#print("frame number:{:}".format(s_fn))
+		v6d = self.v6simo[self.v6simo['fN'] == s_fn]
+		#v6d =  v6dd[v6dd['doppler'] < 0.0]
+		#print(v6d)
+		v7d = self.v7simo[self.v7simo['fN'] == s_fn]
+		v8d = self.v8simo[self.v8simo['fN'] == s_fn]
+		v9d = self.v9simo[self.v9simo['fN'] == s_fn]
+		chk = 0
+		if v6d.count != 0:
+			chk = 1
+		return (chk,v6d,v7d,v8d,v9d)
+		
+		
+	'''
+		v6_col_names_rt = ['fN','type','range','azimuth','elv' ,'doppler','sx', 'sy', 'sz'] #ok
+	v7_col_names_rt = ['fN','type','posX','posY','velX','velY','accX','accY','posZ','velZ','accZ','tid']
+	v8_col_names_rt = ['fN','type','targetID']
+	v9_col_names_rt = ['fN','type','snr','noise']
+	
+	# read from file for trace point clouds
+	fileName = ''
+	v6_col_names = ['time','fN','type','range','azimuth','elv' ,'doppler','sx', 'sy', 'sz']
+	v7_col_names = ['time','fN','type','posX','posY','velX','velY','accX','accY','posZ','velZ','accZ','tid']
+	v8_col_names = ['time','fN','type','targetID']
+	v9_col_names = ['fN','type','snr','noise']
+	'''
+	def readFile(self,fileName):
+		#fileName = "pc32021-03-19-10-02-17.csv"  
+		#df = pd.read_csv(fileName, error_bad_lines=False, warn_bad_lines=False) 
+		self.fileName = fileName 
+		#          ['time','fN','type','elv','azimuth','range' ,'doppler','sx', 'sy', 'sz']
+		df = pd.read_csv(self.fileName, names = self.v6_col_names, skiprows = [0,10,11,12],dtype={'fN': int,'elv': float,'azimuth':float,'range':float,'doppler':float,'sx':float,'sy':float,'sz':float}) 
+		df.dropna()
+		#print("------------------- df --------------------shape:{:}".format(df.shape))
+		print(df)
+		print(df.info())
+		#print(df.info(memory_usage="deep")) 
+		
+		v6simOri = df[(df.type == 'v6')]
+		#print("-------------------v6simo------------:{:}".format(v6simOri.shape))
+									 
+		self.v6simo = v6simOri.loc[:,['fN','type','range','azimuth','elv' ,'doppler','sx', 'sy', 'sz']]
+		
+		if len(self.v6simo):
+			self.sim_startFN = df['fN'].values[0]
+			self.sim_stopFN  = df['fN'].values[-1]
+		#print(self.v6simo)
+		
+		self.v6simo['elv'] = self.v6simo['elv'].astype(float, errors = 'raise') 
+		
+		df7 = pd.read_csv(self.fileName, names = self.v7_col_names, skiprows = [0])  
+		
+		v7simc = df7[df7['type'] == 'v7']
+		self.v7simo  = v7simc.loc[:,['fN','posX','posY','velX','velY','accX','accY','posZ','velZ','accZ','tid']]
+		
+		v8simc = df[df['type'] == 'v8']
+		self.v8simo  = v8simc.loc[:,['fN','elv']]
+		#print(self.v8simo)
+		
+		v9simc = df[df['type'] == 'v9']
+		self.v9simo  = v9simc.loc[:,['fN','type','range','azimuth']]
+		self.v9simo.columns = ['fN','type','snr','noise']
+	
+		return (self.v6simo,self.v7simo,self.v8simo,self.v9simo)
 
 
